/* Copyright (C) 2008 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lua.h>
#include <lauxlib.h>
#include <GL/gl.h>
#include <time.h>
#include "flat.h"

@implementation Flat

-(Flat *) init
{
    [super init];
    
    self->color[0] = 1;
    self->color[1] = 1;
    self->color[2] = 1;
    
    return self;
}

-(void) get: (lua_State *)L
{
    const char *k;
    int i;

    k = lua_tostring(L, 2);

    if (!strcmp(k, "color")) {
        lua_newtable(L);
        
        for(i = 0; i < 3; i += 1) {
            lua_pushnumber(L, self->color[i]);
            lua_rawseti(L, -2, i + 1);
        }
    } else {
	[super get: L];
    }
}

-(void) set: (lua_State *)L
{    
    const char *k;
    int i;

    k = lua_tostring(L, 2);

    if (!strcmp(k, "color")) {
        if(lua_istable(L, 3)) {
            for(i = 0 ; i < 3 ; i += 1) {
                lua_rawgeti(L, 3, i + 1);
                self->color[i] = lua_tonumber(L, -1);
                
                lua_pop(L, 1);
            }
        }
    } else {
	[super set: L];
    }
}

-(void)traverse: (lua_State *)L
{
    glUseProgramObjectARB(0);
    glColor3fv (self->color);

    glEnable (GL_DEPTH_TEST);
    glEnable (GL_CULL_FACE);

    [super traverse: L];

    glDisable (GL_DEPTH_TEST);
    glDisable (GL_CULL_FACE);
}

@end

