/* Copyright (C) 2008 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ode/ode.h>
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

#include "spherical.h"

@implementation Spherical

-(Joint *) init
{
    [super init];
    
    self->joint = dJointCreateBall ([self world], NULL);
    self->amotor = dJointCreateAMotor ([self world], NULL);

    self->anchor[0] = 0;
    self->anchor[1] = 0;
    self->anchor[2] = 0;

    self->axes[0][0] = 1;
    self->axes[0][1] = 0;
    self->axes[0][2] = 0;

    self->axes[2][0] = 0;
    self->axes[2][1] = 0;
    self->axes[2][2] = 1;
    
    self->stops[0][0] = -dInfinity;
    self->stops[0][1] = dInfinity;

    self->stops[1][0] = -dInfinity;
    self->stops[1][1] = dInfinity;

    self->stops[2][0] = -dInfinity;
    self->stops[2][1] = dInfinity;

    return self;
}

-(dJointID) amotor
{
    return self->amotor;
}

-(void) get: (lua_State *)L
{
    const char *k;
    int i, j;
    
    k = lua_tostring(L, -1);

    if (!strcmp(k, "anchor")) {
	dJointGetBallAnchor ([self joint], self->anchor);

        lua_newtable(L);
        
        for(j = 0; j < 3; j += 1) {
            lua_pushnumber(L, self->anchor[j]);
            lua_rawseti(L, -2, j + 1);
        }
    } else if (!strcmp(k, "axes")) {
	dJointGetAMotorAxis ([self amotor], 0, self->axes[0]);
	dJointGetAMotorAxis ([self amotor], 1, self->axes[1]);
	dJointGetAMotorAxis ([self amotor], 2, self->axes[2]);

	lua_newtable(L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable(L);
        
	    for(i = 0; i < 3; i += 1) {
		lua_pushnumber(L, self->axes[j][i]);
		lua_rawseti(L, -2, i + 1);
	    }

	    lua_rawseti(L, -2, j + 1);
	}
    } else if (!strcmp(k, "motor")) {
	lua_newtable(L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable(L);
        
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber(L, self->motor[j][i]);
		lua_rawseti(L, -2, i + 1);
	    }

	    lua_rawseti(L, -2, j + 1);
	}
    } else if (!strcmp(k, "stops")) {
	lua_newtable(L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable(L);

	    lua_newtable(L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber(L, self->stops[j][i]);
		lua_rawseti(L, -2, i + 1);
	    }
	    lua_rawseti(L, -2, 1);
	
	    lua_newtable(L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber(L, self->hardness[j][i]);
		lua_rawseti(L, -2, i + 1);
	    }
	    lua_rawseti(L, -2, 2);

	    lua_pushnumber(L, bounce[j]);
	    lua_rawseti(L, -2, 3);

	    lua_rawseti(L, -2, j + 1);
	}
    } else if (!strcmp(k, "tolerance")) {
	lua_newtable(L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_pushnumber(L, self->tolerance[j]);

	    lua_rawseti(L, -2, j + 1);
	}
    } else if (!strcmp(k, "state")) {
	dReal state[6];

	if ([self joint]) {
	    state[0] = dJointGetAMotorAngle ([self amotor], 0);
	    state[1] = dJointGetAMotorAngle ([self amotor], 1);
	    state[2] = dJointGetAMotorAngle ([self amotor], 2);
	    state[3] = dJointGetAMotorAngleRate ([self amotor], 0);
	    state[4] = dJointGetAMotorAngleRate ([self amotor], 1);
	    state[5] = dJointGetAMotorAngleRate ([self amotor], 2);

	    lua_newtable(L);
        
	    for(j = 0 ; j < 6 ; j += 1) {
		lua_pushnumber(L, state[j]);
		
		lua_rawseti(L, -2, j + 1);
	    }
	} else {
	    lua_pushnil(L);
	}
    } else {
	[super get: L];
    }
}

-(void) set: (lua_State *)L
{
    const char *k;
    int i, j;
    
    k = lua_tostring(L, -2);

    if (!strcmp(k, "anchor")) {
        if(lua_istable(L, 3)) {
            for(j = 0 ; j < 3 ; j += 1) {
                lua_rawgeti(L, 3, j + 1);
                self->anchor[j] = lua_tonumber(L, -1);
                
                lua_pop(L, 1);
            }

	    dJointSetBallAnchor ([self joint],
				 self->anchor[0],
				 self->anchor[1],
				 self->anchor[2]);
        }
    } else if (!strcmp(k, "axes")) {
        if(lua_istable(L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti(L, 3, j + 1);
		
		if(lua_istable(L, -1)) {
		    for(i = 0 ; i < 3 ; i += 1) {
			lua_rawgeti(L, -1, i + 1);
			self->axes[j][i] = lua_tonumber(L, -1);
                
			lua_pop(L, 1);
		    }
		}

		lua_pop (L, 1);
	    }

	    if (dJointGetBody ([self amotor], 0) ||
		dJointGetBody ([self amotor], 1)) {
		dJointSetAMotorAxis ([self amotor], 0, 1,
				     self->axes[0][0],
				     self->axes[0][1],
				     self->axes[0][2]);

		dJointSetAMotorAxis ([self amotor], 2, 2,
				     self->axes[2][0],
				     self->axes[2][1],
				     self->axes[2][2]);
	    }
	}
    } else if (!strcmp(k, "motor")) {
        if(lua_istable(L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti(L, 3, j + 1);
		
		if(lua_istable(L, -1)) {
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti(L, -1, i + 1);
			self->motor[j][i] = lua_tonumber(L, -1);
                
			lua_pop(L, 1);
		    }

		    dJointSetAMotorParam ([self amotor],
					  dParamVel + dParamGroup * j,
					  self->motor[j][0]);
		    
		    dJointSetAMotorParam ([self amotor],
					  dParamFMax + dParamGroup * j,
					  self->motor[j][1]);
		}

		lua_pop (L, 1);
	    }
	}
    } else if (!strcmp(k, "stops")) {
        if(lua_istable(L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti(L, 3, j + 1);
		
		if(lua_istable(L, -1)) {
		    lua_rawgeti(L, -1, 1);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti(L, -1, i + 1);
		
			self->stops[j][i] = lua_tonumber(L, -1);
		
			lua_pop(L, 1);
		    }
		    lua_pop(L, 1);

		    lua_rawgeti(L, -1, 2);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti(L, -1, i + 1);

			self->hardness[j][i] = lua_tonumber(L, -1);
			
			lua_pop(L, 1);
		    }
		    lua_pop(L, 1);
	
		    lua_rawgeti(L, -1, 3);
		    bounce[j] = lua_tonumber(L, -1);
		    lua_pop(L, 1);


		    /* Resetting the self->stops makes sure that lo remains
		       smaller than hi between calls. */

		    dJointSetAMotorParam ([self amotor],
					  dParamLoStop + dParamGroup * j,
					  -dInfinity);
		    
		    dJointSetAMotorParam ([self amotor],
					  dParamHiStop + dParamGroup * j,
					  dInfinity);
		    
		    dJointSetAMotorParam ([self amotor],
					  dParamLoStop + dParamGroup * j,
					  self->stops[j][0]);

		    dJointSetAMotorParam ([self amotor],
					  dParamHiStop + dParamGroup * j,
					  self->stops[j][1]);

		    dJointSetAMotorParam ([self amotor],
					  dParamStopCFM + dParamGroup * j,
					  self->hardness[j][0]);

		    dJointSetAMotorParam ([self amotor],
					  dParamStopERP + dParamGroup * j,
					  self->hardness[j][1]);

		    dJointSetAMotorParam ([self amotor],
					  dParamBounce + dParamGroup * j,
					  bounce[j]);
		}

		lua_pop (L, 1);
	    }
	}
    } else if (!strcmp(k, "tolerance")) {
        if(lua_istable(L, 3)) {
            for(j = 0 ; j < 3 ; j += 1) {
                lua_rawgeti(L, 3, j + 1);
                self->tolerance[j] = lua_tonumber(L, -1);
                
                lua_pop(L, 1);
            }

	    dJointSetAMotorParam ([self amotor],
				  dParamCFM + dParamGroup * j,
				  self->tolerance[j]);
        }
    } else if (!strcmp(k, "bodies")) {
	dBodyID a, b;
	
	[super set: L];
	
	dJointSetBallAnchor ([self joint],
			     self->anchor[0],
			     self->anchor[1],
			     self->anchor[2]);

	a = dJointGetBody ([self joint], 0);
	b = dJointGetBody ([self joint], 1);
	dJointAttach ([self amotor], a, b);
	dJointSetAMotorMode (self->amotor, dAMotorEuler);

	dJointSetAMotorAxis ([self amotor], 0, 1,
			      self->axes[0][0],
			      self->axes[0][1],
			      self->axes[0][2]);

	dJointSetAMotorAxis ([self amotor], 2, 2,
			      self->axes[2][0],
			      self->axes[2][1],
			      self->axes[2][2]);
    }
}

@end
